/** \file iso_com_tcp_receiver_protocol.h
 *
 *  Created by Daniel Bisig on 5/22/09.
 */

#ifndef _iso_com_tcp_receiver_protocol_h_
#define _iso_com_tcp_receiver_protocol_h_

#include "iso_com_network_receiver_protocol.h"
#include <QThread>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>

namespace iso
{

namespace com
{

class TCPReceiverProtocol;

class TCPClientConnection2
{
public:
	TCPClientConnection2(int pClientSocket, const struct sockaddr_in& pClientAddress, unsigned int pMaxPacketSize);
	~TCPClientConnection2();

	bool bufferAvailable() const;
	void removeAvailableBuffer();
	const ByteBuffer& buffer() const throw (ComException);
	
	void receive( const MessageParser& pParser ) throw (ComException);

protected:
	TCPReceiverProtocol* mReceiverProtocol;
	const MessageParser* mParser; 

	int mSocket; ///\brief client socket
	struct sockaddr_in mAddress; ///\brief client address
	bool mConnected;

	char* mReceiveBuffer;
	
	PendingByteBuffer* mPendingBuffer; /// \brief pending buffer
	ByteBuffer* mAvailableBuffer; ///\brief available buffer
	unsigned int mMaxPacketSize;
	
	QMutex mLock;
};

class TCPReceiverProtocol : public NetworkReceiverProtocol, public QThread
{
public:
	/**
	\brief parameter based constructor
	\param pReceiverPort target host port messages are sent to
	\exception ComException failed to create sender
	*/
	TCPReceiverProtocol(unsigned int pReceiverPort) throw (ComException);

	/**
	\brief destructor
	*/
	~TCPReceiverProtocol();

	void removeAvailableBuffer();
	void removeAllBuffers();
	
	void run() throw (ComException);
	void receive( const MessageParser& pParser ) throw (ComException);
	
protected:
	/**
	\brief default constructor
	*/
	TCPReceiverProtocol();
	
	int mSocket; ///\brief receiver socket
	struct sockaddr_in mReceiverSocketAddress;
	unsigned int mMaxClientCount;
	QVector< TCPClientConnection2* > mTCPClientConnections; ///\brief active client connections
};

};

};

#endif