/** \file iso_com_udp_sender_protocol.h
 *
 *  Created by Daniel Bisig on 5/21/09.
 */

#ifndef _iso_com_udp_sender_protocol_h_
#define _iso_com_udp_sender_protocol_h_

#include "iso_com_network_sender_protocol.h"
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>

namespace iso
{

namespace com
{

class UDPSenderProtocol : public NetworkSenderProtocol
{
public:
	/**
	\brief parameter based constructor
	\param pReceiverPort target host port messages are sent to
	\exception ComException failed to create sender
	*/
	UDPSenderProtocol(unsigned int pReceiverPort) throw (ComException);
	
	/**
	\brief parameter based constructor
	\param pReceiverAddress target host address messages are sent to
	\param pReceiverPort target host port messages are sent to
	\exception ComException failed to create sender
	*/
	UDPSenderProtocol(const base::String& pReceiverAddress, unsigned int pReceiverPort) throw (ComException);

	/**
	\brief destructor
	*/
	~UDPSenderProtocol();
	
	/**
	\brief send byte buffer
	\param pByteBuffer byte buffer to be sent
	\param pParser parser for transforming byte buffer into network packets
	\exception ComException failed to send byte buffer
	*/	
	void send( const ByteBuffer& pByteBuffer, const MessageParser& pParser ) throw (ComException);
	
protected:
	/**
	\brief default constructor
	*/
	UDPSenderProtocol();
	
	int mSocket; ///\brief sender socket
	struct sockaddr_in mSocketAddress; ///\brief receiver socket address

};

};

};

#endif