/** \brief iso_data_subtract_function.h
 *
 *  Created by Daniel Bisig on 3/12/09.
 */

#ifndef _iso_data_subtract_function_h_
#define _iso_data_subtract_function_h_

#include "iso_data_value_manager.h"
#include "iso_data_function_manager.h"

namespace iso
{

namespace data
{

#pragma mark definition

class SubtractFunction : public ValueFunction
{
public:
	static const base::String sName;
	
	SubtractFunction();
	~SubtractFunction();

	template<class ValueType> void registerValueType( const ValueType& pValueType );
	template<class ValueType> bool checkValueType( const ValueType& pValueType );	
	template<class ValueType> static void apply(VoidValue& pValue);
	template<class ValueType> static void apply(VoidValue& pValue1, const VoidValue& pValue2);
	template<class ValueType> static void apply(VoidValue& pValue1, const VoidValue& pValue2, VoidValue& pValue3);
};

#pragma mark implementation

template<class ValueType> 
void
SubtractFunction::registerValueType( const ValueType& pValueType )
{
	base::String mapId = typeid( ValueType ).name();
	
	//std::cout << "register value type " << mapId.toStdString() << "\n";
	
	if(mUnaryFunctionMap.contains(mapId) == false) mUnaryFunctionMap[ mapId ] = &( apply<ValueType> );
	if(mBinaryFunctionMap.contains(mapId) == false) mBinaryFunctionMap[ mapId ] = &( apply<ValueType> );
	if(mTernaryFunctionMap.contains(mapId) == false) mTernaryFunctionMap[ mapId ] = &( apply<ValueType> );	
}

template<class ValueType> 
bool 
SubtractFunction::checkValueType( const ValueType& pValueType )
{
	base::String mapId = typeid( ValueType ).name();
	return mUnaryFunctionMap.contains(mapId);
}

template<class ValueType>
void
SubtractFunction::apply(VoidValue& pValue)
{
	//std::cout << "Typed Unary SubtractFunction apply\n";
	
	TypedValue<ValueType>* typedValue = dynamic_cast< TypedValue<ValueType>* > (&pValue);
	
	if( typedValue == NULL ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("value mismatch"), __FILE__, __FUNCTION__, __LINE__ ) );	
	
	//std::cout << "val " << typedValue->value() << "\n";
}

template<class ValueType>
void
SubtractFunction::apply(VoidValue& pValue1, const VoidValue& pValue2)
{
	//std::cout << "Typed Binary SubtractFunction apply\n";
	
	TypedValue<ValueType>* typedValue1 = dynamic_cast< TypedValue<ValueType>* > (&pValue1);
	TypedValue<ValueType>* typedValue2 = dynamic_cast< TypedValue<ValueType>* > (&pValue2);
	
	if( typedValue1 == NULL || typedValue2 == NULL) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("value mismatch"), __FILE__, __FUNCTION__, __LINE__ ) );	
	
	//std::cout << typedValue1->value() << " - " << typedValue2->value() << " -> ";
	
	typedValue1->value() = typedValue1->value() - typedValue2->value();
	
	//std::cout << typedValue1->value() << "\n";
}	

template<class ValueType>
void
SubtractFunction::apply(VoidValue& pValue1, const VoidValue& pValue2, VoidValue& pValue3)
{
	//std::cout << "Typed Ternary SubtractFunction apply\n";

	TypedValue<ValueType>* typedValue1 = dynamic_cast< TypedValue<ValueType>* > (&pValue1);
	TypedValue<ValueType>* typedValue2 = dynamic_cast< TypedValue<ValueType>* > (&pValue2);
	TypedValue<ValueType>* typedValue3 = dynamic_cast< TypedValue<ValueType>* > (&pValue3);
	
	if( typedValue1 == NULL || typedValue2 == NULL || typedValue3 == NULL) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("value mismatch"), __FILE__, __FUNCTION__, __LINE__ ) );	
	
	//std::cout << typedValue1->value() << " - " << typedValue2->value() << " -> ";
	
	typedValue3->value() = typedValue1->value() - typedValue2->value();
	
	//std::cout << typedValue3->value() << "\n";
}	

};

};

#endif