/** \file iso_data_typed_value.h
 * \class iso::data::TypedValue template class for storing an arbitary data type
 * \brief template class for storing an arbitary data type
 *
 *  Created by Daniel Bisig on 3/13/09.
 */

#ifndef _iso_data_typed_value_h_
#define _iso_data_typed_value_h_

#include "iso_data_void_value.h"

namespace iso
{

namespace data 
{

#pragma mark TypedValue (definition)

template <class Type>
class TypedValue : public VoidValue
{
public:
	TypedValue();
	TypedValue( Type pValue );	
	TypedValue( const TypedValue& pValue );	
	~TypedValue();	
	
	Type& value();	
	const Type& value() const;	
	template <class Type2> const TypedValue& operator=( const TypedValue<Type2>& pValue ) throw (base::Exception);	
	operator Type() throw (base::Exception);		
	TypedValue* copy() const;
	base::String info() const;
	
	friend std::ostream& operator<< (std::ostream & pOstream, const TypedValue& pValue)
	{
		pOstream << pValue.info().toStdString(); 
		
		return pOstream;
	}		
};

#pragma mark TypedValue (implementation)

template <class Type>
TypedValue<Type>::TypedValue( )
	: VoidValue( Type() )
{}

template <class Type>
TypedValue<Type>::TypedValue( Type pValue )
	: VoidValue( pValue )
{}

template <class Type>
TypedValue<Type>::TypedValue( const TypedValue& pValue )
	: VoidValue( *( static_cast<Type*>( pValue.mValue ) ) )
{}

template <class Type>
TypedValue<Type>::~TypedValue()
{
	delete static_cast<Type*>( mValue );
}

template <class Type>
Type&
TypedValue<Type>::value()
{
	return *( static_cast<Type*>( mValue ) );
}

template <class Type>
const Type&
TypedValue<Type>::value() const
{
	return *( static_cast<Type*>( mValue ) );
}

template <class Type>
template <class Type2>
const TypedValue<Type>& 
TypedValue<Type>::operator=( const TypedValue<Type2>& pValue ) throw (base::Exception)
{
	*( static_cast<Type*>( mValue ) ) = pValue.value();
	
	return *this;
}		

template <class Type>
TypedValue<Type>::operator Type() throw (base::Exception)
{
	return *( static_cast<Type*>( mValue ) );
}

template <class Type>
TypedValue<Type>* 
TypedValue<Type>::copy() const
{
	return new TypedValue( *this );
}

template <class Type>	
base::String 
TypedValue<Type>::info() const
{
	base::String infoString = base::String("value type %1\n")
	.arg( mType->name() );

	infoString.append( static_cast<Type*>( mType )->info() );

	return infoString;
}

};

};

#endif