/** \file iso_data_value_manager.h
 *  \class iso::data::ValueManager singleton helper class for storing arbitrary data types
 *  \brief singleton helper class for storing arbitrary data types
 *
 *  Created by Daniel Bisig on 25/2/09.
 */

#ifndef _iso_data_value_manager_h_
#define _iso_data_value_manager_h_

#include "iso_data_singleton.h"
#include "iso_data_void_value.h"
#include "iso_data_typed_value.h"
#include "iso_data_values.h"
#include <iso_base/iso_base_exception.h>
#include <iostream>

namespace iso
{

namespace data
{

#pragma mark ValueManager (definition)

typedef void (*AssignFunctionPtr)(VoidValue&, const VoidValue& );

class ValueManager
{
public:
	static ValueManager& get();
	static void destroy();

	void assign( VoidValue& pValue1, const VoidValue& pValue2) throw(base::Exception);
	void assign( Values& pValues, const VoidValue& pValue) throw(base::Exception);
	void assign( Values& pValues1, const Values& pValues2) throw(base::Exception);
	
	template<class ValueType> void registerCastType();
	template<class ValueType1, class ValueType2> void registerCastType();
	template<class ValueType1, class ValueType2> static void assignTypedValue(VoidValue& pValue1, const VoidValue& pValue2) throw(base::Exception);	

	void print();
	
	QHash< base::String, AssignFunctionPtr > mAssignFunctionMap;

protected:
	ValueManager();
	~ValueManager();

	static ValueManager* sInstance;

	//QHash< base::String, AssignFunctionPtr > mAssignFunctionMap;
};

#pragma mark ValueManager (implementation)

template<class ValueType>
void
ValueManager::registerCastType()
{
	base::String mapId = base::String("%1_%2").arg( typeid( ValueType ).name() ).arg( typeid( ValueType ).name() );
	
	if(mAssignFunctionMap.contains(mapId) == false) 
	{
//		std::cout << "register cast type " << mapId.toStdString() << "\n";
	
		mAssignFunctionMap[ mapId ] = &( assignTypedValue<ValueType, ValueType> );	
	}
}

template<class ValueType1, class ValueType2> 
void 
ValueManager::registerCastType()
{
	base::String mapId = base::String("%1_%2").arg( typeid( ValueType1 ).name() ).arg( typeid( ValueType2 ).name() );
	
	if(mAssignFunctionMap.contains(mapId) == false) 
	{
		//std::cout << "register cast type " << mapId.toStdString() << "\n";
	
		mAssignFunctionMap[ mapId ] = &( assignTypedValue<ValueType1, ValueType2> );	
	}
}

template<class ValueType1, class ValueType2>  
void 
ValueManager::assignTypedValue(VoidValue& pValue1, const VoidValue& pValue2) throw(base::Exception)
{
	if( pValue1.match< ValueType1 >() == false || pValue2.match< ValueType2 >() == false ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied data types don't match"), __FILE__, __FUNCTION__, __LINE__ ) );

	TypedValue<ValueType1>* typedValue1 = static_cast< TypedValue<ValueType1>* > (&pValue1);
	const TypedValue<ValueType2>* typedValue2 = static_cast< const TypedValue<ValueType2>* > (&pValue2);
	
	typedValue1->value() = typedValue2->value();
}

//template<class ValueType1, class ValueType2>  
//void 
//ValueManager::assignTypedValue(VoidValue& pValue1, const VoidValue& pValue2) throw(base::Exception)
//{
//	TypedValue<ValueType1>* typedValue1 = dynamic_cast< TypedValue<ValueType1>* > (&pValue1);
//	const TypedValue<ValueType2>* typedValue2 = dynamic_cast< const TypedValue<ValueType2>* > (&pValue2);
//	
//	if( typedValue1 == NULL || typedValue2 == NULL ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied data types don't match"), __FILE__, __FUNCTION__, __LINE__ ) ); 
//	
//	typedValue1->value() = typedValue2->value();
//}
		
};

};

#endif
