/** \file iso_data_void_value.h
 * \class iso::data::VoidValue abstract single value container class
 * \brief abstract single value container class
 *
 * stores a value of an arbitrary data type as void pointer along with information about the data type
 *
 *  Created by Daniel Bisig on 3/13/09.
 */

#ifndef _iso_data_void_value_h_
#define _iso_data_void_value_h_

#include "iso_data_types.h"
#include <iso_base/iso_base_exception.h>
#include <typeinfo>

namespace iso
{

namespace data
{

#pragma mark VoidValue (definition)

class VoidValue
{
public:
	template <class Type> VoidValue();
	template <class Type> VoidValue( const Type& pValue );
	~VoidValue();
	
	const std::type_info* type() const;
	virtual VoidValue* copy() const = 0;
	virtual const VoidValue& operator=( const VoidValue& pValue ) throw (base::Exception);
	template <class Type> operator Type() const throw (base::Exception);
	template <class Type> inline bool match() const;
	bool match( const VoidValue& pValue ) const;
	template <class Type> void get( Type& pValue ) const throw (base::Exception);
	template <class Type> Type& value() throw (base::Exception);
	template <class Type> const Type& value() const throw (base::Exception);
	base::String info() const;
	
	friend std::ostream& operator<< (std::ostream & pOstream, const VoidValue& pValue)
	{
		pOstream << pValue.info().toStdString(); 
		
		return pOstream;
	}		
	
protected:
	void* mValue;
	const std::type_info* mType;
};

#pragma mark VoidValue (implementation)

template <class Type>
VoidValue::VoidValue( )
{
	Type* type = new Type();
	mType = &typeid( *type );
	mValue = static_cast<void*>( type );
}

template <class Type>
VoidValue::VoidValue( const Type& pValue )
{
	mType = &typeid( pValue );
	mValue = static_cast<void*>( new Type( pValue ) );
}

template <class Type> 
VoidValue::operator Type() const throw (base::Exception)
{
	//if( &typeid(Type) != mType ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(Type).name() ).arg( mType->name() ), __FILE__, __FUNCTION__, __LINE__ ));	
	if( match<Type>() == false ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(Type).name() ).arg( mType->name() ), __FILE__, __FUNCTION__, __LINE__ ));

	return *( static_cast<Type*>( mValue ) );
}

template <class Type>
bool
VoidValue::match() const
{
//	if( &typeid(Type) == mType ) return true;
//	return false;

	if( typeid(Type) == (*mType) || base::String( typeid(Type).name() ) == base::String( mType->name() ) ) return true;
	return false;
}
			
template <class Type> 
void
VoidValue::get( Type& pValue ) const throw (base::Exception)
{
//	if( &typeid(pValue) != mType ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(pValue).name() ).arg( mType->name() ), __FILE__, __FUNCTION__, __LINE__ ));
	if( match<Type>() == false ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(pValue).name() ).arg( mType->name() ), __FILE__, __FUNCTION__, __LINE__ ));

	pValue = *( static_cast<Type*>( mValue ) );
}

template <class Type> 
Type&
VoidValue::value() throw (base::Exception)
{
//	if( &typeid(Type) != mType ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(Type).name() ).arg( mType->name() ), __FILE__, __FUNCTION__, __LINE__ ));
	if( match<Type>() == false ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(Type).name() ).arg( mType->name() ), __FILE__, __FUNCTION__, __LINE__ ));

	return *( static_cast<Type*>( mValue ) );
}

template <class Type> 
const Type&
VoidValue::value() const throw (base::Exception)
{
//	if( &typeid(Type) != mType ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(Type).name() ).arg( mType->name() ), __FILE__, __FUNCTION__, __LINE__ ));
	if( match<Type>() == false ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(Type).name() ).arg( mType->name() ), __FILE__, __FUNCTION__, __LINE__ ));

	base::String pTypeName( typeid(Type).name() );
	base::String mTypeName( mType->name() );

	std::cout << "pTypeName " << pTypeName.toStdString() << " mTypeName " << mTypeName.toStdString() << "\n";
	std::cout << "mType == Type: " << ( (*mType) ==typeid(Type) ) << "\n";
	std::cout << "name(): pTypeName == mTypeName " << ( mTypeName == pTypeName ) << "\n";
	std::cout << "QString: pTypeName == mTypeName " << ( typeid(Type).name() == mType->name() ) << "\n";

	return *( static_cast<Type*>( mValue ) );
}

};

};

#endif