/** \file iso_font.h
 *
 *  Created by Daniel Bisig on 7/10/09.
 *
 */

#ifndef _iso_font_h_
#define _iso_font_h_

#include "iso_font_types.h"
#include "iso_font_exception.h"
#include <iso_math/iso_math_vector2.h>

#include <ft2build.h>
#include <freetype/ftglyph.h>
#include FT_FREETYPE_H
#include <QTextCodec>

namespace iso
{

namespace font
{

struct LetterMetrics
{
	math::Vector2<int> mSize;
	math::Vector2<int> mHOrigin;
	math::Vector2<int> mVOrigin;
	int mHAdvance;
	int mVAdvance;
};

class Font
{
public:
	Font();
	Font( const base::String& pFontName, const base::String& pFontFile, unsigned int pFontHeight ) throw (FontException);
	~Font();

	const base::String& fontName() const;
	FT_Glyph glyph( const QChar& pChar ) throw (FontException);
	const LetterMetrics& letterMetrics( const QChar& pChar ) throw (FontException);
	math::Vector2<int> letterKerning( const QChar& pLeftLetter, const QChar& pRightLetter ) throw (FontException);
	math::Vector2<int> textSize( const base::String& pText, const math::Vector2<float>& pSpacing, bool pKerning) throw (FontException);
	void nextLetterPosition( math::Vector2<int>& pCursorPos, math::Vector2<int>& pLetterPos, unsigned int& pLetterIndex, const base::String& pText, const math::Vector2<float>& pSpacing, bool pKerning) throw (FontException);

protected:
	void init( const base::String& pFontName, const base::String& pFontFile, unsigned int pFontHeight ) throw (FontException);
	
	void storeLetterMetrics( const QChar& pLetter ) throw (FontException);

	base::String mFontName;
	int mFontHeight;
	FT_Face mFontFace; 
	
	QHash< QChar, LetterMetrics* > mLetterMetrics;
	
};

};

};

#endif