/** \file iso_font_manager.h
 *
 *  Created by Daniel Bisig on 7/10/09.
 *
 */

#ifndef _iso_font_manager_h_
#define _iso_font_manager_h_

#include "iso_font_types.h"
#include "iso_font_exception.h"
#include "iso_font_bitmap_font.h"
#include "iso_font_vector_font.h"
#include <iso_data/iso_data_singleton.h>
#include <iso_data/iso_data_buffer.h>
#include <iso_geom/iso_geom_geometry_group.h>

#include <ft2build.h>
#include FT_FREETYPE_H
#include <QHash>

namespace iso
{

namespace font
{

class FontManager : public data::Singleton<FontManager>
{
public:
	FontManager() throw (FontException);
	~FontManager();
	
	BitmapFont& bitmapFont( const base::String& pFontName ) throw (FontException);
	VectorFont& vectorFont( const base::String& pFontName ) throw (FontException);
	
	void createBitmapFont( const base::String& pFontName, const base::String& pFontFile, unsigned int pFontHeight ) throw (FontException);
	void createVectorFont( const base::String& pFontName, const base::String& pFontFile, unsigned int pSubdivisionCount ) throw (FontException);
	data::Buffer<unsigned char>* bitmapText( const base::String& pFontName, const base::String& pText, const math::Vector2<float>& pSpacing, bool pKerning ) throw (FontException);
	geom::GeometryGroup* vectorText( const base::String& pFontName, const base::String& pText, const math::Vector2<float>& pSize, const math::Vector2<float>& pSpacing, bool pKerning ) throw (FontException);
	
	FT_Library& fontLibrary();

protected:
	FT_Library mFontLibrary; ///\brief freetype font library
	
	QHash< base::String, BitmapFont* > mBitmapFonts;
	QHash< base::String, VectorFont* > mVectorFonts;

};

};

};

#endif