/** \file iso_geom_geometry.h
 *
 *  Created by Daniel Bisig on 5/14/07.
 */

#ifndef _iso_geom_geometry_h_
#define _iso_geom_geometry_h_

#include "iso_geom_types.h"
#include <iso_math/iso_math_vector.h>

namespace iso
{

namespace geom
{

class Geometry
{
public:
	/**
	\brief create geometry
	\param pDim dimension
	*/
	Geometry(unsigned int pDim);
	
	/**
	\brief destructor
	*/
	virtual ~Geometry();

	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos() = 0;
	
	/**
	\brief return dimension
	\return dimension
	*/
	unsigned int dim() const;
	
	/**
	\brief return minimum position
	\return minimum position
	*/
	const math::Vector<real>& minPos() const;
	
	/**
	\brief return maximum position
	\return maximum position
	*/
	const math::Vector<real>& maxPos() const;
	
	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	virtual void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint) = 0;

protected:
	/**
	\brief default constructor
	*/
	Geometry();
	
	unsigned int mDim; /// \brief dimension
	math::Vector<real> mMinPos; /// \brief minimum position
	math::Vector<real> mMaxPos; /// \brief maximum position
	
};

};

};

#endif