/** \file iso_geom_hermite_spline_alg.h
 *
 *  Created by Daniel Bisig on 4/1/07.
 */

#ifndef _iso_geom_hermite_spline_alg_h_
#define _iso_geom_hermite_spline_alg_h_

#include "iso_geom_spline_alg.h"

namespace iso
{

namespace geom
{

class HermiteSplineAlg : public SplineAlg
{
public:
	/**
	\brief create hermite spline
	\param p_dimension dimension of spline 
	\param p_subdivisionCount number of subdivisions
	*/
	HermiteSplineAlg(unsigned int p_dimension, unsigned int p_subdivisionCount);
	
	/**
	\brief destructor
	*/
	~HermiteSplineAlg();
	
	/**
	\brief create a copy of a spline alg
	\return copy of spline alg
	*/
	virtual SplineAlg* copy() const;	

	/**
	\brief set subdivision count
	\param p_subdivisionCount subdivision count
	*/
	virtual void setSubdivisionCount(unsigned int p_subdivisionCount);

	/**
	\brief create spline
	\param p_controlPoints control points
	\param p_splinePoints resulting spline points
	\param b_closed whether spline should be closed or not
	*/
	virtual void createSpline(QVector< math::Vector<real> >& p_controlPoints, QVector< math::Vector<real> >& p_splinePoints, bool b_closed = false);

protected:
	/**
	\brief default constructor
	*/
	HermiteSplineAlg();

	/**
	\brief precalculate polynomial values
	*/
	virtual void preCalc();
};

};

};

#endif