/** \file iso_geom_plane_tools.h
 *
 *  Created by Daniel Bisig on 12/25/08.
 */

#ifndef _iso_geom_plane_tools_h_
#define _iso_geom_plane_tools_h_

#include <iso_data/iso_data_singleton.h>
#include <iso_math/iso_math_vector3.h>
#include "iso_geom_plane.h"
#include "iso_geom_ray.h"

namespace iso
{

namespace geom
{

class PlaneTools : public data::Singleton<PlaneTools>
{
public:
	/**
	\brief calculcate closest point to plane
	\param pRefPoint search point
	\param pResultPoint resulting closest point
	\param pPlane plane
	*/
	void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResultPoint, const Plane& pPlane);
	
	/**
	\brief calculate line intersection between two planes
	\param pPlane1 first plane
	\param pPlane2 second plane
	\param pResultLine resulting line intersection
	*/
	void intersect(const Plane& pPlane1, const Plane& pPlane2, Ray& pResultRay);

	/**
	\brief calculate intersection between ray and plane
	\param pRay ray
	\param pPlane plane
	\param pResultPoint resulting point intersection
	*/
	void intersect(Ray& pRay, const Plane& pPlane1, math::Vector<real>& pResultPoint);

	PlaneTools();
	~PlaneTools();

};


};

};

#endif