/** \file iso_geom_triangle.h
 *
 *  Created by Daniel Bisig on 9/17/07.
 */

#ifndef _iso_geom_triangle_h_
#define _iso_geom_triangle_h_

#include "iso_geom_types.h"
#include "iso_geom_geometry.h"
#include <iso_math/iso_math_vector.h>

namespace iso
{

namespace geom
{

class Triangle : public Geometry
{
public:
	Triangle();
	Triangle(const math::Vector<real>& pV1, const math::Vector<real>& pV2, const math::Vector<real>& pV3);
	Triangle(const Triangle& pTriangle);
	~Triangle();

	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();
	
	inline math::Vector<real>& v0();
	inline math::Vector<real>& v1();
	inline math::Vector<real>& v2();

	inline const math::Vector<real>& v0() const;
	inline const math::Vector<real>& v1() const;
	inline const math::Vector<real>& v2() const;
	
	void set(unsigned int pVIndex, real pX, real pY, real pZ);
	void set(unsigned int pVIndex, const math::Vector<real>& pV);
	void set(const math::Vector<real>& pV1, const math::Vector<real>& pV2, const math::Vector<real>& pV3);
	void set(const Triangle& pTriangle);
	const Triangle& operator=( const Triangle& pTriangle );

	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	virtual void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);

protected:

	math::Vector<real> mV[3];
};

math::Vector<real>& 
Triangle::v0()
{
	return mV[0];
}

math::Vector<real>&
Triangle::v1()
{
	return mV[1];
}

math::Vector<real>&
Triangle::v2()
{
	return mV[2];
}

const math::Vector<real>& 
Triangle::v0() const
{
	return mV[0];
}

const math::Vector<real>&
Triangle::v1() const
{
	return mV[1];
}

const math::Vector<real>&
Triangle::v2() const
{
	return mV[2];
}

};

};

#endif