/** \file iso_geom_triangle_tools.h
 *
 *  Created by Daniel Bisig on 9/17/07.
 */

#ifndef _iso_geom_triangle_tools_h_
#define _iso_geom_triangle_tools_h_

#include <iso_math/iso_math_vector3.h>
#include "iso_geom_triangle.h"

namespace iso
{

namespace geom
{

class TriangleTools
{
public:
	static TriangleTools& get();
	static void destroy();

	/**
	\brief calculate distance of point to triangle
	\param pRefPoint search point
	\param pTriangle triangle
	\param pSP resulting S parameter
	\param pST resulting T parameter
	\return distance
	*/
	real distance(const math::Vector<real>& pRefPoint, const Triangle& pTriangle, real& pSP, real& pTP);
	
	/**
	\brief calculate point within triangle
	\param pSP resulting S parameter
	\param pST resulting T parameter
	\param pTriangle triangle
	\param pPoint resulting point
	*/
	void point(real pSP, real pTP, const Triangle& pTriangle, math::Vector<real>& pPoint);	

	/**
	\brief calculcate closest point to triangle
	\param pRefPoint search point
	\param pResultPoint resulting closest point
	\param pTriangle triangle
	*/
	void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResultPoint, const Triangle& pTriangle);

protected:
	TriangleTools();
	~TriangleTools();

	static TriangleTools* sTriangleTools;

};


};

};

#endif