/** \file iso_gl.h
 *
 *  Created by Daniel Bisig on 8/11/07.
 */

#ifndef _iso_gl_h_
#define _iso_gl_h_

#include "iso_gl_exception.h"
#ifdef _linux_
#define GL_GLEXT_PROTOTYPES
#include <GL/glu.h>
#include <GL/glext.h>
#else
#include <OpenGL/glu.h>
#include <OpenGL/glext.h>
#endif

namespace iso
{

namespace gl
{

class Shader;	
class Program;
	
class GL
{
public:
	/**
	\brief return singleton instance
	\return singleton instance
	*/
	static GL& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy();
	
	/**
	\brief obtain OpenGL version
	\param pMajor major version number
	\param pMinor minor version number
	*/
	void version(int& pMajor, int& pMinor) throw (GLException);
	
	/**
	 \brief return maximum supported number of simultaneous texture units
	 returns maximum supported number of simultaneous texture units
	 */
	int maxTextureUnits(); 

	void clearError();

	/**
	\brief check for open gl errors
	\exception throws an exception if there has been an gl error, none otherwise
	*/
	void error() throw (GLException);
	
	/**
	 \brief return info log for shader
	 \param pShader shader
	 \returns info log for shader
	 */
	base::String infoLog( Shader* pShader );

	/**
	 \brief return info log for progra,
	 \param pProgram progra,
	 \returns info log for program
	 */
	base::String infoLog( Program* pProgram );
	
	/**
	\brief convert error number to string
	\param pError error number
	*/
	base::String errorToString( GLenum pError );

protected:
	static GL* sGL; ///\brief singleton instance
	
	/**
	/brief default constructor
	*/
	GL();
	
	/**
	\brief destructor
	*/
	~GL();

};

};

};

#endif