/** \file iso_gl_exception.h
 *  iso_gl
 *
 *  Created by Daniel Bisig on 1/19/10.
 *
 */

#ifndef _iso_gl_exception_h_
#define _iso_gl_exception_h_

#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace gl
{

class GLException : public base::Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	GLException(base::String pType, base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	GLException(base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);

	/**
	\brief copy constructor
	\param pException exception
	*/
	GLException( const Exception& pException );
	
	/**
	\brief destructor
	*/
	~GLException();
	
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const GLException& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
		
protected:	
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();

	/**
	\brief default constructur
	*/
	GLException();	
};

};

};

#endif