/** \file iso_gl_shader.h
 *
 *  Created by Daniel Bisig on 8/12/07.
 */

#ifndef _iso_gl_shader_h_
#define _iso_gl_shader_h_

#include "iso_gl.h"
#include "iso_gl_types.h"
#include <QVector>

namespace iso
{

namespace gl
{

enum ShaderType
{
	VertexShader,
	FragmentShader
};

class Shader
{
public:
	/**
	\brief create shader
	\param pShaderType shader type
	*/
	Shader(ShaderType pShaderType) throw (GLException);
	
	/**
	\brief destructor
	*/	
	~Shader();

	/**
	\brief return true if shader is loaded, false otherwise
	*/
	inline bool loaded() const;
	
	/**
	\brief return true if shader is compiled, false otherwise
	*/
	inline bool compiled() const;
	
	/**
	\brief compile shader
	*/
	void compile() throw (GLException);	
	
	/**
	\brief return shader id
	\return shader id
	*/
	GLuint id() const;
	
	/**
	\brief load shader source code
	\param pFileName source code file name
	\remarks fails if file is not found
	\return true if file could be loaded, false otherwise
	*/
	void load(base::String pFileName) throw (GLException);
	
	/**
	\brief delete shader
	*/
	void destroy();

	/**
	\brief print shader info log
	*/
	void printInfoLog() const;

protected:
	/**
	\brief default constructor
	*/
	Shader() throw (GLException);

	GLuint mId; ///\brief shader id 
	ShaderType mType; ///\brief shader type
	char* mSourceCode;
	bool mLoaded;
	bool mCompiled;
};

bool 
Shader::compiled() const
{
	return mCompiled;
}

};

};

#endif