/** \file iso_glsl.h
 *
 *  Created by Daniel Bisig on 8/11/07.
 */

#ifndef _iso_glsl_h_
#define _iso_glsl_h_

#include "iso_gl_exception.h"
#include "iso_gl_shader.h"
#include "iso_gl_program.h"

#include <QHash>
#ifdef _linux_
#include <GL/glu.h>
#include <GL/glext.h>
#else
#include <OpenGL/glu.h>
#include <OpenGL/glext.h>
#endif

namespace iso
{

namespace gl
{

class GLSL
{
public:
	/**
	\brief return singleton instance
	\return singleton instance
	*/
	static GLSL& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy();
	
	/**
	\brief obtain OpenGL version
	\param pMajor major version number
	\param pMinor minor version number
	*/
	void version(int& pMajor, int& pMinor) throw (GLException);
	
	/**
	\brief register a shader
	\param pShader shader
	*/
	void registerShader(Shader& pShader);
	
	/**
	\brief register a program
	\param pProgram program
	*/
	void registerProgram(Program& pProgram);
	
	/**
	\brief print shader info log
	\param pShaderId shader id
	*/
	void printShaderInfoLog(GLuint pShaderId);

	/**
	\brief print program info log
	\param pProgramId shader id
	*/
	void printProgramInfoLog(GLuint pProgramId);

protected:
	static GLSL* sGLSL; ///\brief singleton instance
	
	/**
	/brief default constructor
	*/
	GLSL();
	
	/**
	\brief destructor
	*/
	~GLSL();
	
	/**
	\brief shader registry
	*/
	QHash< GLuint, Shader* > mShaders;
	
	/**
	\brief program registry
	*/
	QHash< GLuint, Program* > mPrograms;

};

};

};

#endif