/** \file iso_space_alg.h
 *
 *  Created by Daniel Bisig on 6/7/09.
 */

#ifndef _iso_space_alg_h_
#define _iso_space_alg_h_

#include "iso_space_types.h"
#include "iso_space_exception.h"
#include "iso_space_proxy_object.h"
#include <iso_math/iso_math_vector.h>
#include <QVector>

namespace iso
{

namespace space
{

class SpaceAlg
{
public:
	SpaceAlg( unsigned int mDim );
	SpaceAlg( const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos );
	virtual ~SpaceAlg();
	
	bool fixedSize() const;
	unsigned int dim() const;
	const math::Vector<real>& minPos() const;
	const math::Vector<real>& maxPos() const;
	
	virtual void resize(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos) throw (SpaceException);
	virtual void updateStructure( QVector<SpaceProxyObject*>& pObjects ) throw (SpaceException);
	virtual void updateNeighbors( QVector<SpaceProxyObject*>& pObjects ) throw (SpaceException);

	/**
	\brief print space alg information
	*/
	base::String info() const;

  	/**
	\brief retrieve textual space info
	\param pOstream output text stream
	\param pSpace space
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const SpaceAlg& pAlg )
    {
		pOstream << pAlg.info().toStdString(); 
					
        return pOstream;
    }; 
	
protected:
	SpaceAlg();

	bool mFixedSize;
	math::Vector<real> mMinPos;
	math::Vector<real> mMaxPos;

};

};

};

#endif