/** \file iso_space_alg_ann.h
 *
 *  Created by Daniel Bisig on 6/9/09.
 */
 
#ifndef _iso_space_alg_ann_h_
#define _iso_space_alg_ann_h_

#include "iso_space_alg.h"
#include <ANN/ANN.h>
#include <ANN/ANNx.h>

namespace iso
{

namespace space
{

class ANNAlg : public SpaceAlg
{
public:
	ANNAlg(unsigned int pDim);
	ANNAlg(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos);
	~ANNAlg();

	void updateStructure( QVector< SpaceProxyObject* >& pObjects ) throw (SpaceException);
	void updateNeighbors( QVector< SpaceProxyObject* >& pObjects ) throw (SpaceException);
	
	/**
	\brief obtain textual ntree information
	\return String containing textual ntree information
	*/
	base::String info() const;

	/**
	\brief retrieve textual ntree information
	\param pOstream output stream
	\param pTree ntree
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const ANNAlg& pAlg)
	{
		pOstream << pAlg.info().toStdString(); 
		
		return pOstream;
	}

protected:
	ANNAlg();
	
	ANNkd_tree* mTree; // search structure
	ANNpointArray mDataPts; // data points
	QVector< SpaceProxyObject* >* mNeighborObjects; //  helper data structure (all space objects whose position is stored in the tree stucture)
};

};

};

#endif