/** \file iso_space_alg_ntree.h
 *
 *  Created by Daniel Bisig on 6/7/09.
 *
 */

#ifndef _iso_space_alg_ntree_h_
#define _iso_space_alg_ntree_h_

#include "iso_space_alg.h"
#include "iso_space_ntree.h"
#include "iso_space_ntree_visitor.h"

namespace iso
{

namespace space
{

class NTreeNode;

class NTreeAlg : public SpaceAlg
{
friend class NTreeVisitor;

public:
	NTreeAlg(unsigned int pDim);
	NTreeAlg(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos);
	~NTreeAlg();

	void resize(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos) throw (SpaceException);
	void updateStructure( QVector< SpaceProxyObject* >& pObjects ) throw (SpaceException);
	void updateNeighbors( QVector< SpaceProxyObject* >& pObjects ) throw (SpaceException);
	
	/**
	\brief obtain textual ntree information
	\return String containing textual ntree information
	*/
	base::String info() const;

	/**
	\brief retrieve textual ntree information
	\param pOstream output stream
	\param pTree ntree
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const NTreeAlg& pAlg)
	{
		pOstream << pAlg.info().toStdString(); 
		
		return pOstream;
	}

protected:
	NTreeAlg();

	/**
	\brief NTree space partitioning instance
	*/
	NTree mTree;
	
	/**
	\brief NTreeVisitor visitor for Ntree
	*/
	NTreeVisitor mTreeVisitor;
};

};

};

#endif