/** \file iso_space_cluster_analyzer.h
 *  \class iso::space::SpaceClusterAnalyzer detects clusters among space objects
 *  \brief detects clusters among space objects
 *
 *  Created by Daniel Bisig on 5/30/08.
 */
 
 #include "iso_space_objects_analyzer.h"
 
 namespace iso
 {
 
 namespace space
 {
 
 class SpaceClusterAnalyzer : public SpaceObjectsAnalyzer
 {
 public:
 	/**
	\brief default constructor
	*/
	SpaceClusterAnalyzer();

	/**
	\brief parameter based constructor
	\param pName of space data analyzer
	*/
	SpaceClusterAnalyzer(base::String pName);
	
	/**
	\brief destructor
	*/	
	~SpaceClusterAnalyzer();
 
	/**
	\brief add space object group
	\param pGroupName name of space object group
	*/	
	virtual void addObjectGroup( const base::String& pGroupName );
 
 	/**
	\brief add space object
	\param pGroupIndex space object group index
	\param pSpaceObject space object
	*/
	virtual void addObject( unsigned int pGroupIndex, const SpaceObject* pSpaceObject );
	
	/**
	\brief add space objects
	\param pGroupIndex space object group index
	\param pSpaceObjects vector of space objects
	*/
	virtual void addObjects( unsigned int pGroupIndex, QVector< const SpaceObject* >& pSpaceObjects );	

	/**
	\brief add space object
	\param pGroupName space object group name
	\param pSpaceObject space objects
	*/
	virtual void addObject( const base::String& pGroupName, const SpaceObject* pSpaceObject );

	/**
	\brief add space objects
	\param pGroupName space object group name
	\param pSpaceObjects vector of space objects
	*/
	virtual void addObjects( const base::String& pGroupName, QVector< const SpaceObject* >& pSpaceObjects );
 
 	/**
	\brief perform analysis
	*/
	virtual void analyze();
 
 protected:

 };
 
 };
 
 }; 

