/**	\file iso_space_ntree_node_pool.h
 *	\class iso::space::NTreeNodePool pool of ntree nodes
 *	\brief pool of ntree nodes
 *
 *
 *  Created by Daniel Bisig on 3/29/07.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_space_ntree_node_pool_h_
#define _iso_space_ntree_node_pool_h_

#include <iso_base/iso_base_types.h>
#include <ostream>
#include <QVector>

namespace iso
{

namespace space
{

class NTreeNode;

class NTreeNodePool
{
public:
	/**
	\brief create pool of ntree nodes
	\param pDim dimension of nodes
	*/
	NTreeNodePool(unsigned int pDim);

	/**
	\brief create pool of ntree nodes
	\param pDim dimension of nodes
	\param pStartPoolSize number of nodes that are initially in the pool
	\param pPoolSizeIncrement size by which the pool increases whenever it runs out of nodes
	*/
	NTreeNodePool(unsigned int pDim, unsigned int pStartPoolSize, unsigned int pPoolSizeIncrement = sPoolSizeIncrement);
	
	/**
	\brief destructor
	*/
	virtual ~NTreeNodePool();
	
	/**
	\brief retrieve node from pool
	*/
	NTreeNode* retrieve();
	
	/**
	\brief release node into pool
	\param pNode node to be released
	*/
	void release(NTreeNode* pNode);
	
	/**
	\brief obtain textual node pool information
	\return String containing node pool information
	*/
	base::String info() const;

	/**
	\brief retrieve textual ntree node pool information
	\param pOstream output stream
	\param pPool ntree node pool
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const NTreeNodePool& pPool)
	{
		pOstream << pPool.info().toStdString(); 
		
		return pOstream;
	}
	
protected:
	/**
	\brief default constructor
	*/
	NTreeNodePool();

	/**
	\brief default initial pool size
	*/
	static unsigned int sStartPoolSize;

	/**
	\brief default pool size increment
	*/
	static unsigned int sPoolSizeIncrement;
	
	/**
	\brief node dimension
	*/
	unsigned int mDim;
	
	/**
	\brief size by which the pool increases whenever it runs out of events
	*/
	unsigned int mPoolSizeIncrement;

	/**
	\brief vector of unused ntree nodes
	*/
	QVector<NTreeNode*> mNodes;
};

};

};

#endif