/**	\file iso_space_ntree_visitor.h
 *	\class iso::space::NTreeVisitor visitor for parameter space tree
 *	\brief visitor for parameter space tree
 *
 *
 *  Created by Daniel Bisig on 3/29/07.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
 #ifndef _iso_space_ntree_visitor_h_
 #define _iso_space_ntree_visitor_h_
 
 #include "iso_space_ntree.h"
 #include "iso_space_ntree_node.h"
 
 namespace iso
 {
 
 namespace space
 {
 
 class NeighborPool;
 class SpaceProxyObject;
 
 class NTreeVisitor
 {
 public:
 	NTreeVisitor(unsigned int pDimension);
	~NTreeVisitor();

	void createNodePool();

	void buildTree(NTree& pTree, QVector<SpaceProxyObject*>& pObjects);
	void buildTree(NTree& pTree, NTreeNode* pNode);
	void updateTree(NTree& pTree, QVector<SpaceProxyObject*>& pObjects);
	void updateTree(NTree& pTree, NTreeNode* pNode);
	
	void calcNeighbors(NTree& pTree, QVector<SpaceProxyObject*>& pObjects);
	void calcNeighbors(NTreeNode* pNode, QVector<SpaceProxyObject*>& pObjects);
	
	
	//void calcNeighbors(NTree& pTree);
	//void calcNeighbors(NTreeNode* pNode);
	void calcNeighbors(NTreeNode* pNode, SpaceProxyObject* pObject);
	void clearTree(NTree& pTree);
	void clearTree(NTreeNode* pNode);

	void printTree(const NTree& pTree) const;
	void printTree(const NTreeNode* pNode) const;
 
	/**
	\brief print ntree information
	\param pTree ntree
	*/
	base::String info(const NTree& pTree) const;

	/**
	\brief print ntree node information
	\param pNode ntree node
	*/
	base::String info(const NTreeNode* pNode) const;

protected:
	NTreeVisitor();
	
	/**
	\brief dimension
	*/
	unsigned int mDim;
	
	/**
	\brief temporaey node center position
	*/
	math::Vector<real> mCenterPos;
	
	/**
	\brief temporary node minimum position
	*/
	math::Vector<real> mMinPos;
	
	/**
	\brief temporary node maximum position
	*/
	math::Vector<real> mMaxPos;
	
	/**
	\brief pool of nodes for building an ntree
	*/
	NTreeNodePool* mNodePool;
 
 };
 
 };
 
 };
 
 #endif
 

