/**	\file iso_xml_tools.h
 *  \class iso::xml::XMLTools helper functions for xml parsing
 *	\brief helper functions for xml parsing
 * 
 *  Created by Daniel Bisig on 8/19/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
#ifndef _iso_xml_tools_h_
#define _iso_xml_tools_h_

#include "iso_xml_types.h"
#include "iso_xml_exception.h"

#include <iostream>
#include<xercesc/dom/DOM.hpp>
#include<xercesc/dom/DOMDocument.hpp>
#include<xercesc/dom/DOMDocumentType.hpp>
#include<xercesc/dom/DOMElement.hpp>
#include<xercesc/dom/DOMImplementation.hpp>
#include<xercesc/dom/DOMImplementationLS.hpp>
#include<xercesc/dom/DOMNodeIterator.hpp>
#include<xercesc/dom/DOMNodeList.hpp>
#include<xercesc/dom/DOMText.hpp>
#include<xercesc/framework/LocalFileFormatTarget.hpp>
#include<xercesc/parsers/XercesDOMParser.hpp>
#include<xercesc/util/XMLUni.hpp>

namespace iso
{

namespace xml
{

class XMLTools
{
public:
	/**
	\brief get singleton instance
	\return singleton instance
	*/
	static XMLTools& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy();
	
	/**
	\brief create DOM element
	\param pElementName tag name of element
	\param pDocument DOM document
	\return DOM element
	*/
	xercesc::DOMElement* createElement(const base::String& pElementName, xercesc::DOMDocument* pDocument);
	
	/**
	\brief check whether DOM element contains attribute
	\param pElement DOM element
	\param pAttribute attribute name
	\return true, if attribute is present in attribute list of DOM element, false otherwise
	*/
	bool containsAttribute(xercesc::DOMElement* pElement, const base::String& pAttribute);
	
	/**
	\brief add attribute to DOM element
	\param pElement DOM element
	\param pAttribute attribute name
	\param pValue atribute value
	*/
	void addAttribute(xercesc::DOMElement* pElement, const base::String& pAttribute, const base::String& pValue);
	
	/**
	\brief create DOM parser from file
	\param pFile file name
	\return DOM parser
	\exception Exception parse error or DOM exception
	*/
	xercesc::XercesDOMParser* load(base::String pFile) throw (XMLException);

protected:
	/**
	\brief default constructor
	*/
	XMLTools() throw (XMLException);
	
	/**
	\brief destructor
	*/
	~XMLTools() throw (XMLException);

	/**
	\brief singleton instance
	*/
	static XMLTools* sXMLTools;
};

};

};

#endif